type OdataIdentifier = string;
type PrimaryExpr = FunctionExpr | ArrayExpr | Primitive | MemberExpr;
interface UnaryExpr {
    type: string;
    value: unknown;
}
interface Primitive extends UnaryExpr {
    type: 'Primitive';
    value: string | number | boolean | null;
}
interface BinaryExpr {
    type: string;
    left: unknown;
    right: unknown;
}
interface MemberExpr extends UnaryExpr {
    type: 'MemberExpr';
    value: OdataIdentifier;
}
interface FunctionExpr {
    type: 'FunctionExpr';
    name: string;
    arguments: PrimaryExpr[];
}
interface InExpr extends BinaryExpr {
    type: 'InExpr';
    left: PrimaryExpr;
    right: ArrayExpr;
}
interface EqExpr extends BinaryExpr {
    type: 'EqExpr';
    left: PrimaryExpr;
    right: PrimaryExpr;
}
interface NeExpr extends BinaryExpr {
    type: 'NeExpr';
    left: PrimaryExpr;
    right: PrimaryExpr;
}
interface GtExpr extends BinaryExpr {
    type: 'GtExpr';
    left: PrimaryExpr;
    right: PrimaryExpr;
}
interface GeExpr extends BinaryExpr {
    type: 'GeExpr';
    left: PrimaryExpr;
    right: PrimaryExpr;
}
interface LtExpr extends BinaryExpr {
    type: 'LtExpr';
    left: PrimaryExpr;
    right: PrimaryExpr;
}
interface LeExpr extends BinaryExpr {
    type: 'LeExpr';
    left: PrimaryExpr;
    right: PrimaryExpr;
}
type RelationalExpr = FunctionExpr | InExpr | OrExpr | EqExpr | NeExpr | GtExpr | GeExpr | LtExpr | LeExpr;
interface AndExpr extends BinaryExpr {
    type: 'AndExpr';
    left: RelationalExpr;
    right: RelationalExpr;
}
interface OrExpr extends BinaryExpr {
    type: 'OrExpr';
    left: AndExpr | RelationalExpr;
    right: AndExpr | RelationalExpr;
}
interface ArrayExpr extends UnaryExpr {
    type: 'ArrayExpr';
    value: Primitive[];
}
declare enum OrderByDirection {
    ASC = "asc",
    DESC = "desc"
}
interface OrderByItem {
    type: 'OrderByItem';
    expr: PrimaryExpr;
    dir: OrderByDirection;
}
type Filter = OrExpr | AndExpr | RelationalExpr;
declare const parseFilter: (query: string) => Filter;
type OrderBy = OrderByItem[];
declare const parseOrderBy: (query: string) => OrderBy;

export { OrderByDirection, parseFilter, parseOrderBy };
export type { AndExpr, ArrayExpr, BinaryExpr, EqExpr, Filter, FunctionExpr, GeExpr, GtExpr, InExpr, LeExpr, LtExpr, MemberExpr, NeExpr, OdataIdentifier, OrExpr, OrderBy, OrderByItem, PrimaryExpr, Primitive, RelationalExpr, UnaryExpr };
